#include <iostream>
#include <set>
#include <sstream>
#include <string>
#include <windows.h>

using namespace std;

struct mac
{
	mac(const unsigned char x[6]){
		memcpy(addr, x, 6 * sizeof(unsigned char));
	}
	mac(std::string s)
	{
		std::stringstream ss;
		ss << s;
		unsigned char a,b;
		for (int i=0;i<6;++i)
		{
			if (i) ss >> a;
			ss >> a >> b;
			addr[i]  = ((a>='0'&&a<='9')?(a-'0'):((a|0x20)+10-'a'))<<4;
			addr[i] |= ((b>='0'&&b<='9')?(b-'0'):((b|0x20)+10-'a'));
		}
	}
	bool operator <(const mac & other) const
	{
		for (int x=0;x<6;++x)
		{
			if (addr[x] < other.addr[x]) return true;
			if (addr[x] > other.addr[x]) return false;
		}
		return false;
	}
	operator std::string () const
	{
		std::string s;
		for (int x=0;x<6;++x)
		{
			int i;
			if (x) s += ":";
			s += (i = ((addr[x] & 0xf0) >> 4)) < 10 ? ('0' + i) : ('A' - 10 + i);
			s += (i = (addr[x] & 0x0f)) < 10 ? ('0' + i) : ('A' - 10 + i);
		}
		return s;
	}
	unsigned char addr[6];
};

/*
 * NDAS functionality - interface with the code provided in NDASUSER to generate keys.
 */

typedef int  (__stdcall *PFVERIFY)(
	const char *id, const char *pw
);
typedef int  (__stdcall *PFMAKE)(
	const byte *pMac, char *pOutID, 
    char *pOutKey, const unsigned *pdwCode1, 
    const unsigned *pdwCode2
);

PFVERIFY pfVerify = 0;
PFMAKE   pfMake   = (PFMAKE) 0x67AADC80; // 3.11.1327.0

void print_ndas_info(const mac &m)
{
	char szID[22];
	char szKey[6];
	memset(szID, 0, sizeof(szID));
	memset(szKey, 0, sizeof(szKey));
	unsigned long magic1[] = { 0x2f563245, 0x53384aec, 0xeb0f4e1e, 0x0c1502733 };
	unsigned long magic2[] = { 0xffff01cd };

	int hr = (*pfMake)(m.addr, szID, szKey, (const unsigned *) magic1, (const unsigned *) magic2);
	if (!hr) 
	{
		std::cout << "Generation of NDAS id for MAC " << static_cast<std::string>(m) << " failed.\n";
		return;
	}
	std::cout << "MAC       : " << static_cast<std::string>(m) << "\n" << 
				 "NDAS ID   : " << szID << "\n" << 
				 "Write Key : " << szKey << "\n";
	if (!pfVerify(szID, szKey))
		std::cout << "WARNING: Verify returns: " << pfVerify(szID, szKey) << "\n";

}

int main(int argc, char *argv[])
{
    std::set<mac>                 seen_macs;
    std::set<mac>::const_iterator sit;
    
	std::string libname("ndasuser.dll");
	HMODULE hLib = LoadLibrary(libname.c_str());
	if (!hLib)
	{
		std::cout << "Failed to load NDASUSER dll from " << libname << "\n";
		std::cout << "Error code was " << GetLastError();
		exit(1);
	}

	pfVerify = (PFVERIFY) GetProcAddress(hLib, (const char *) 11);
	if (!pfVerify)
	{
		std::cout << "Failed to get NDASUSER functions\n";
		return EXIT_FAILURE;
	}

	if (argc > 1)
	{
		for (int i=1; i<argc; ++i) 
		{
			std::cout << "Forcing MAC [" << argv[i] << "]" << std::endl;
			seen_macs.insert(mac(std::string(argv[i])));
		}
	}
	
	if (seen_macs.size() == 0) 
    	std::cout << "Usage: ndasinfo mac1 mac2 ..." << std::endl;
	for (sit = seen_macs.begin(); sit != seen_macs.end(); ++sit)
	{
		std::cout << "--------[Displaying NDAS Info]--------" << std::endl;
		try 
		{
			print_ndas_info(*sit);
		} 
		catch (...)
		{
			std::cout << "**Error while calling into NDASUSER.DLL**" << std::endl;
		}
	}

    system("PAUSE");
    return EXIT_SUCCESS;
}
